<?php

namespace App\Commands\Concerns;

use App\Git\CloneTask;
use App\Projects\ProjectBase;
use GitWrapper\GitException;
use Throwable;

trait ClonesProjects
{
    protected function cloneProject(ProjectBase $projectBase): void
    {
        try {
            $this->task(
                'Cloning ' . $projectBase->label,
                new CloneTask($projectBase)
            );
        } catch (GitException $exception) {
            $this->error(
                "Failed to clone the project ($projectBase->url). See error message below"
            );
            $this->output->block($this->tidyExceptionMessage($exception), 'git', 'error');

            return;
        }
    }

    private function tidyExceptionMessage(Throwable $exception): array
    {
        return explode(PHP_EOL, str_replace("\r", '', $exception->getMessage()));
    }
}
