<?php

namespace App\Database;

use App\Projects\ProjectName;
use LaravelZero\Framework\Commands\Command;

class ConnectionSettings
{
    /**
     * @var \LaravelZero\Framework\Commands\Command
     */
    private $output;

    public function __construct(Command $output)
    {
        $this->output = $output;
    }

    public static function request(Command $output): array
    {
        return (new self($output))->getConnectionSettings();
    }

    private function getConnectionSettings(): array
    {
        $config = $this->askConnectionSettings();
        if ($config['database'] === null || $config['username'] === null || $config['password'] === null) {
            $this->output->error(
                'You must answer all the questions.' .
                PHP_EOL
            );

            return $this->getConnectionSettings();
        }

        return $config;
    }

    private function askConnectionSettings(): array
    {
        $defaultDbName = ProjectName::retrieve();

        return $this->makeConfig(
            $this->output->ask('What is the database host?', '127.0.0.1'),
            $this->output->ask('What is the database port?', '3306'),
            $this->output->ask('What is the database name?', $this->sanitizeDatabaseName($defaultDbName)),
            $this->output->ask('What is the database username?', 'root'),
            $this->output->ask('What is the database password?', '')
        );
    }

    private function makeConfig(?string $host, ?string $port, ?string $database, ?string $username, ?string $password): array
    {
        return [
            'driver' => 'mysql',
            'host' => $host,
            'port' => $port,
            'database' => $this->sanitizeDatabaseName($database),
            'username' => $username,
            'password' => $password,
            'unix_socket' => '',
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => true,
            'engine' => null,
        ];
    }

    /**
     * @param string|null $database
     * @return string|string[]|null
     */
    private function sanitizeDatabaseName(?string $database)
    {
        return preg_replace('/(?![\w_-])./', '', $database);
    }
}
