<?php

namespace App\Database\Env;

use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class Writer
{
    public const ENV_FILE = '.env';

    /**
     * @var array
     */
    private $envMapping;

    public function __construct(array $envMapping)
    {
        $this->envMapping = $envMapping;
    }

    public function put(array $config)
    {
        $envFile = Storage::disk('project')->get(self::ENV_FILE);

        $envFile = preg_replace(
            [
                "/{$this->envMapping['dbHost']}=.*/",
                "/{$this->envMapping['dbPort']}=.*/",
                "/{$this->envMapping['dbName']}=.*/",
                "/{$this->envMapping['dbUsername']}=.*/",
                "/{$this->envMapping['dbPassword']}=.*/",
            ],
            [
                $this->envMapping['dbHost'] . '=' . $config['host'],
                $this->envMapping['dbPort'] . '=' . $config['port'],
                $this->envMapping['dbName'] . '=' . $this->wrapIfSpaces($config['database']),
                $this->envMapping['dbUsername'] . '=' . $this->wrapIfSpaces($config['username']),
                $this->envMapping['dbPassword'] . '=' . $this->wrapIfSpaces($config['password']),
            ],
            $envFile
        );

        Storage::disk('project')->put(self::ENV_FILE, $envFile);
    }

    private function wrapIfSpaces(string $value): string
    {
        if (Str::contains($value, ' ')) {
            return '"' . $value . '"';
        }

        return $value;
    }
}
