<?php

namespace App\Projects;

use App\Git\Cloneable;
use ErrorException;
use Illuminate\Support\Str;

class ProjectBase implements Cloneable
{
    /**
     * @var string
     */
    private $type;

    /**
     * @var string
     */
    private $label;

    /**
     * @var string
     */
    private $url;

    public function __construct(string $type, string $label, string $url)
    {
        $this->type = $type;
        $this->label = $label;
        $this->url = $url;
    }

    public static function fromArray($attributes): self
    {
        return new self(
            $attributes['type'],
            $attributes['label'],
            $attributes['url']
        );
    }

    public function getFolder(): string
    {
        return Str::slug($this->type);
    }

    public function getCloneUrl(): string
    {
        return $this->url;
    }

    /**
     * @return mixed
     */
    public function __get(string $name)
    {
        throw_if(
            property_exists($this, $name) === false,
            new ErrorException('Undefined property: ' . self::class . "::$name")
        );

        return $this->$name;
    }

    public function __isset(string $name): bool
    {
        return property_exists($this, $name);
    }
}
